import { NodeType, } from './types/types';
import { toSingularAction } from './singular';
import { nodeToType } from './structure';
// ============== single ==============
export var single_actionTypes = function (nodeName) {
    return {
        get: "GET_" + nodeName.toUpperCase(),
        setAll: "SET_" + nodeName.toUpperCase(),
        patch: "PATCH_" + nodeName.toUpperCase(),
    };
};
// ============== queue ==============
export var queue_actionTypes = function (nodeName) {
    return {
        get: "GET_" + nodeName.toUpperCase(),
        setAll: "SET_" + nodeName.toUpperCase(),
        push: "PUSH_" + toSingularAction(nodeName),
        pop: "POP_" + toSingularAction(nodeName),
        clear: "CLEAR_" + nodeName.toUpperCase(),
    };
};
// ============== collection ==============
export var collection_actionTypes = function (nodeName) {
    return {
        get: "GET_" + nodeName.toUpperCase(),
        setAll: "SET_" + nodeName.toUpperCase(),
        set: "SET_" + toSingularAction(nodeName),
        add: "ADD_" + toSingularAction(nodeName),
        patch: "PATCH_" + toSingularAction(nodeName),
        delete: "DELETE_" + toSingularAction(nodeName),
    };
};
// ============== grouped list ==============
export var groupedList_actionTypes = function (nodeName) {
    return {
        get: "GET_" + nodeName.toUpperCase(),
        setAll: "SET_" + nodeName.toUpperCase(),
        set: "SET_" + toSingularAction(nodeName),
        add: "ADD_" + toSingularAction(nodeName),
        patch: "PATCH_" + toSingularAction(nodeName),
        delete: "DELETE_" + toSingularAction(nodeName),
        getItems: "GET_" + nodeName.toUpperCase() + "_ITEMS",
        setItems: "SET_" + nodeName.toUpperCase() + "_ITEMS",
        pushItem: "PUSH_" + nodeName.toUpperCase() + "_ITEM",
        popItem: "POP_" + nodeName.toUpperCase() + "_ITEM",
        clearItems: "CLEAR_" + nodeName.toUpperCase() + "_ITEMS",
        pushManyItems: "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS",
    };
};
// ============== from store structure ==============
export var generateActionTypesForStore = function (storeState) {
    var output = {};
    var keys = Object.keys(storeState);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var value = storeState[key];
        var type = nodeToType(value);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = single_actionTypes(String(key));
                break;
            case NodeType.QUEUE_NODE:
                output[key] = queue_actionTypes(String(key));
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = collection_actionTypes(String(key));
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = groupedList_actionTypes(String(key));
                break;
        }
    }
    return output;
};
var GET_VERBS = ['get'];
var LOCAL_VERBS = [
    'setAll',
    'set',
    'setMany',
    'pushMany',
    'setItems',
    'pushManyItems',
];
export var generateActionTypesDictionaryForStore = function (storeState) {
    var output = {};
    var storeActionTypes = generateActionTypesForStore(storeState);
    var nodeKeys = Object.keys(storeActionTypes);
    var _loop_1 = function (nodeName) {
        var bag = storeActionTypes[nodeName];
        Object.keys(bag).forEach(function (apiVerb) {
            var actionType = bag[apiVerb];
            output[actionType] = {
                verb: apiVerb,
                nodeName: nodeName,
            };
            if (LOCAL_VERBS.includes(apiVerb)) {
                output[actionType].isLocal = true;
            }
            if (GET_VERBS.includes(apiVerb)) {
                output[actionType].isGet = true;
            }
        });
    };
    for (var _i = 0, nodeKeys_1 = nodeKeys; _i < nodeKeys_1.length; _i++) {
        var nodeName = nodeKeys_1[_i];
        _loop_1(nodeName);
    }
    return output;
};
