import { __assign } from "tslib";
import { NodeType, } from './types/types';
import { toSingularAction } from './singular';
import { nodeToType } from './structure';
export var SIGNATURE = {
    '@@redux-store-generator/AUTO_GENERATED_ACTION': true,
};
// ============== single ==============
export var single_patchAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "PATCH_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var single_setAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var single_getAction = function (nodeName, extra) { return function () {
    return __assign({ type: "GET_" + nodeName.toUpperCase() }, extra);
}; };
export var single_all = function (nodeName, extra) {
    return {
        get: single_getAction(nodeName, extra),
        setAll: single_setAction(nodeName, extra),
        patch: single_patchAction(nodeName, extra),
    };
};
// ============== queue ==============
export var queue_getAction = function (nodeName, extra) { return function (payload) {
    if (payload === void 0) { payload = {}; }
    return __assign({ type: "GET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var queue_setAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var queue_pushAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "PUSH_" + toSingularAction(nodeName), payload: payload }, extra);
}; };
export var queue_pushManyAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "PUSH_MANY_" + nodeName.toUpperCase(), payload: {
            items: payload,
        } }, extra);
}; };
export var queue_popAction = function (nodeName, extra) { return function () {
    return __assign({ type: "POP_" + toSingularAction(nodeName) }, extra);
}; };
export var queue_clearAction = function (nodeName, extra) { return function () {
    return __assign({ type: "CLEAR_" + nodeName.toUpperCase() }, extra);
}; };
export var queue_all = function (nodeName, extra) {
    return {
        get: queue_getAction(nodeName, extra),
        setAll: queue_setAction(nodeName, extra),
        push: queue_pushAction(nodeName, extra),
        pop: queue_popAction(nodeName, extra),
        clear: queue_clearAction(nodeName, extra),
        pushMany: queue_pushManyAction(nodeName, extra),
    };
};
// ============== collection ==============
export var collection_getAction = function (nodeName, extra) { return function (payload) {
    if (payload === void 0) { payload = {}; }
    return __assign({ type: "GET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var collection_setAllAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var collection_addAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "ADD_" + toSingularAction(nodeName), payload: __assign({}, payload) }, extra);
}; };
export var collection_setAction = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "SET_" + toSingularAction(nodeName), payload: __assign({ id: id }, payload) }, extra);
}; };
export var collection_patchAction = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "PATCH_" + toSingularAction(nodeName), payload: __assign({ id: id }, payload) }, extra);
}; };
export var collection_deleteAction = function (nodeName, extra) { return function (id) {
    return __assign({ type: "DELETE_" + toSingularAction(nodeName), payload: {
            id: id,
        } }, extra);
}; };
export var collection_setManyAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_MANY_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var collection_all = function (nodeName, extra) {
    return {
        get: collection_getAction(nodeName, extra),
        setAll: collection_setAllAction(nodeName, extra),
        set: collection_setAction(nodeName, extra),
        add: collection_addAction(nodeName, extra),
        patch: collection_patchAction(nodeName, extra),
        delete: collection_deleteAction(nodeName, extra),
        setMany: collection_setManyAction(nodeName, extra),
    };
};
// ============== grouped list ==============
export var groupedList_getAction = function (nodeName, extra) { return function (payload) {
    if (payload === void 0) { payload = {}; }
    return __assign({ type: "GET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var groupedList_setAllAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var groupedList_addAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "ADD_" + toSingularAction(nodeName), payload: __assign({}, payload) }, extra);
}; };
export var groupedList_setAction = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "SET_" + toSingularAction(nodeName), payload: __assign({ id: id }, payload) }, extra);
}; };
export var groupedList_patchAction = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "PATCH_" + toSingularAction(nodeName), payload: __assign({ id: id }, payload) }, extra);
}; };
export var groupedList_deleteAction = function (nodeName, extra) { return function (id) {
    return __assign({ type: "DELETE_" + toSingularAction(nodeName), payload: {
            id: id,
        } }, extra);
}; };
export var groupedList_setManyAction = function (nodeName, extra) { return function (payload) {
    return __assign({ type: "SET_MANY_" + nodeName.toUpperCase(), payload: payload }, extra);
}; };
export var groupedList_getItems = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "GET_" + nodeName.toUpperCase() + "_ITEMS", payload: __assign({ id: id }, payload) }, extra);
}; };
export var groupedList_setItems = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "SET_" + nodeName.toUpperCase() + "_ITEMS", payload: {
            id: id,
            payload: {
                items: payload,
            },
        } }, extra);
}; };
export var groupedList_pushItem = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "PUSH_" + nodeName.toUpperCase() + "_ITEM", payload: __assign({ id: id }, payload) }, extra);
}; };
export var groupedList_popItem = function (nodeName, extra) { return function (id) {
    return __assign({ type: "POP_" + nodeName.toUpperCase() + "_ITEM", payload: {
            id: id,
        } }, extra);
}; };
export var groupedList_clearItems = function (nodeName, extra) { return function (id) {
    return __assign({ type: "CLEAR_" + nodeName.toUpperCase() + "_ITEMS", payload: {
            id: id,
        } }, extra);
}; };
export var groupedList_pushManyItems = function (nodeName, extra) { return function (id, payload) {
    return __assign({ type: "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS", payload: {
            id: id,
            items: payload,
        } }, extra);
}; };
export var groupedList_all = function (nodeName, extra) {
    return {
        get: groupedList_getAction(nodeName, extra),
        setAll: groupedList_setAllAction(nodeName, extra),
        set: groupedList_setAction(nodeName, extra),
        add: groupedList_addAction(nodeName, extra),
        patch: groupedList_patchAction(nodeName, extra),
        delete: groupedList_deleteAction(nodeName, extra),
        setMany: groupedList_setManyAction(nodeName, extra),
        getItems: groupedList_getItems(nodeName, extra),
        setItems: groupedList_setItems(nodeName, extra),
        pushItem: groupedList_pushItem(nodeName, extra),
        popItem: groupedList_popItem(nodeName, extra),
        clearItems: groupedList_clearItems(nodeName, extra),
        pushManyItems: groupedList_pushManyItems(nodeName, extra),
    };
};
// ============== from store structure ==============
export var generateActionsForStore = function (storeState) {
    var output = {};
    var keys = Object.keys(storeState);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var value = storeState[key];
        var type = nodeToType(value);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = single_all(String(key), SIGNATURE);
                break;
            case NodeType.QUEUE_NODE:
                output[key] = queue_all(String(key), SIGNATURE);
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = collection_all(String(key), SIGNATURE);
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = groupedList_all(String(key), SIGNATURE);
                break;
        }
    }
    return output;
};
