import { __assign, __spreadArray } from "tslib";
import { toSingularAction } from './singular';
import { nodeToType } from './structure';
import { NodeType } from './types/types';
export var generateSingle = function (nodeName) { return function (state, action) {
    if (state === void 0) { state = {}; }
    switch (action.type) {
        case "SET_" + nodeName.toUpperCase():
            return action.payload;
        case "PATCH_" + nodeName.toUpperCase():
            return __assign(__assign({}, state), action.payload);
        default:
            return state || {};
    }
}; };
export var generateQueue = function (nodeName) { return function (state, action) {
    if (state === void 0) { state = []; }
    var newState;
    switch (action.type) {
        case "SET_" + nodeName.toUpperCase():
            return action.payload;
        case "PUSH_" + toSingularAction(nodeName):
            return __spreadArray(__spreadArray([], state), [action.payload]);
        case "POP_" + toSingularAction(nodeName):
            newState = __spreadArray([], state);
            newState.pop();
            return newState;
        case "CLEAR_" + nodeName.toUpperCase():
            return [];
        case "PUSH_MANY_" + nodeName.toUpperCase():
            var _a = (action.payload || {}).items, items = _a === void 0 ? [] : _a;
            return __spreadArray(__spreadArray([], state), (items || []));
        default:
            return state || [];
    }
}; };
export var generateCollection = function (nodeName) {
    var item = function (state, action) {
        if (state === void 0) { state = {}; }
        switch (action.type) {
            case "SET_" + toSingularAction(nodeName):
                return action.payload;
            case "PATCH_" + toSingularAction(nodeName):
                return __assign(__assign({}, state), action.payload);
            default:
                return state;
        }
    };
    var items = function (state, action) {
        var _a;
        if (state === void 0) { state = {}; }
        var newState;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase():
                return action.payload;
            case "SET_" + toSingularAction(nodeName):
            case "PATCH_" + toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                return __assign(__assign({}, state), (_a = {}, _a[action.payload.id] = item(state[action.payload.id], action), _a));
            case "DELETE_" + toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                newState = __assign({}, state);
                delete newState[action.payload.id];
                return newState;
            case "SET_MANY_" + nodeName.toUpperCase():
                return __assign(__assign({}, state), action.payload);
            default:
                return state || {};
        }
    };
    return items;
};
export var generateGroupedList = function (nodeName) {
    var listItems = function (state, action) {
        var newState;
        var _a = (action.payload || {}).items, items = _a === void 0 ? [] : _a;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
                return items;
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                return __spreadArray(__spreadArray([], state), items);
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
                newState = __spreadArray([], state);
                newState.pop();
                return newState;
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
                return [];
            default:
                return state || [];
        }
    };
    var item = function (state, action) {
        if (state === void 0) { state = {}; }
        switch (action.type) {
            case "SET_" + toSingularAction(nodeName):
                return action.payload;
            case "PATCH_" + toSingularAction(nodeName):
                return __assign(__assign({}, state), action.payload);
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                return __assign(__assign({}, state), { items: listItems(state.items, action) });
            default:
                return state;
        }
    };
    var items = function (state, action) {
        var _a;
        if (state === void 0) { state = {}; }
        var newState;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase():
                return action.payload;
            case "SET_" + toSingularAction(nodeName):
            case "PATCH_" + toSingularAction(nodeName):
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                return __assign(__assign({}, state), (_a = {}, _a[action.payload.id] = item(state[action.payload.id], action), _a));
            case "DELETE_" + toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                newState = __assign({}, state);
                delete newState[action.payload.id];
                return newState;
            case "SET_MANY_" + nodeName.toUpperCase():
                return __assign(__assign({}, state), action.payload);
            default:
                return state || {};
        }
    };
    return items;
};
export var generateReducersForStore = function (storeState) {
    var output = {};
    var keys = Object.keys(storeState);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var type = nodeToType(storeState[key]);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = generateSingle(String(key));
                break;
            case NodeType.QUEUE_NODE:
                output[key] = generateQueue(String(key));
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = generateCollection(String(key));
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = generateGroupedList(String(key));
                break;
        }
    }
    return output;
};
