import { NodeType } from './types/types';
var isQueueNode = function (value) {
    return Array.isArray(value);
};
var isCollectionNode = function (value) {
    if (typeof value !== 'object')
        return false;
    var firstKey = Object.keys(value).pop();
    return value && value[firstKey] && value[firstKey].id === firstKey;
};
var isGroupedListNode = function (value) {
    if (typeof value !== 'object')
        return false;
    var firstKey = Object.keys(value).pop();
    var firstValue = firstKey ? value[firstKey] : null;
    var items = (firstValue || {}).items;
    return isCollectionNode(value) && Array.isArray(items);
};
var isSingleNode = function (value) {
    if (typeof value !== 'object')
        return false;
    return !value['id'];
};
export var nodeToType = function (value) {
    if (isQueueNode(value)) {
        return NodeType.QUEUE_NODE;
    }
    else if (isGroupedListNode(value)) {
        return NodeType.GROUPED_LIST_NODE;
    }
    else if (isCollectionNode(value)) {
        return NodeType.COLLECTION_NODE;
    }
    else if (isSingleNode(value)) {
        return NodeType.SINGLE_NODE;
    }
    return undefined;
};
export var analyzeStructure = function (json) {
    var memo = {};
    return Object.keys(json).reduce(function (output, key) {
        if (nodeToType(json[key])) {
            output[key] = nodeToType(json[key]);
        }
        return output;
    }, memo);
};
export var clearNodes = function (json, nodes) {
    // generating a new object for immutability sake
    return Object.keys(json)
        .filter(function (key) { return !nodes.includes(key); })
        .reduce(function (output, key) {
        output[key] = json[key];
        return output;
    }, {});
};
