"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateReducersForStore = exports.generateGroupedList = exports.generateCollection = exports.generateQueue = exports.generateSingle = void 0;
var tslib_1 = require("tslib");
var singular_1 = require("./singular");
var structure_1 = require("./structure");
var types_1 = require("./types/types");
var generateSingle = function (nodeName) { return function (state, action) {
    if (state === void 0) { state = {}; }
    switch (action.type) {
        case "SET_" + nodeName.toUpperCase():
            return action.payload;
        case "PATCH_" + nodeName.toUpperCase():
            return tslib_1.__assign(tslib_1.__assign({}, state), action.payload);
        default:
            return state || {};
    }
}; };
exports.generateSingle = generateSingle;
var generateQueue = function (nodeName) { return function (state, action) {
    if (state === void 0) { state = []; }
    var newState;
    switch (action.type) {
        case "SET_" + nodeName.toUpperCase():
            return action.payload;
        case "PUSH_" + singular_1.toSingularAction(nodeName):
            return tslib_1.__spreadArray(tslib_1.__spreadArray([], state), [action.payload]);
        case "POP_" + singular_1.toSingularAction(nodeName):
            newState = tslib_1.__spreadArray([], state);
            newState.pop();
            return newState;
        case "CLEAR_" + nodeName.toUpperCase():
            return [];
        case "PUSH_MANY_" + nodeName.toUpperCase():
            var _a = (action.payload || {}).items, items = _a === void 0 ? [] : _a;
            return tslib_1.__spreadArray(tslib_1.__spreadArray([], state), (items || []));
        default:
            return state || [];
    }
}; };
exports.generateQueue = generateQueue;
var generateCollection = function (nodeName) {
    var item = function (state, action) {
        if (state === void 0) { state = {}; }
        switch (action.type) {
            case "SET_" + singular_1.toSingularAction(nodeName):
                return action.payload;
            case "PATCH_" + singular_1.toSingularAction(nodeName):
                return tslib_1.__assign(tslib_1.__assign({}, state), action.payload);
            default:
                return state;
        }
    };
    var items = function (state, action) {
        var _a;
        if (state === void 0) { state = {}; }
        var newState;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase():
                return action.payload;
            case "SET_" + singular_1.toSingularAction(nodeName):
            case "PATCH_" + singular_1.toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                return tslib_1.__assign(tslib_1.__assign({}, state), (_a = {}, _a[action.payload.id] = item(state[action.payload.id], action), _a));
            case "DELETE_" + singular_1.toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                newState = tslib_1.__assign({}, state);
                delete newState[action.payload.id];
                return newState;
            case "SET_MANY_" + nodeName.toUpperCase():
                return tslib_1.__assign(tslib_1.__assign({}, state), action.payload);
            default:
                return state || {};
        }
    };
    return items;
};
exports.generateCollection = generateCollection;
var generateGroupedList = function (nodeName) {
    var listItems = function (state, action) {
        var newState;
        var _a = (action.payload || {}).items, items = _a === void 0 ? [] : _a;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
                return items;
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                return tslib_1.__spreadArray(tslib_1.__spreadArray([], state), items);
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
                newState = tslib_1.__spreadArray([], state);
                newState.pop();
                return newState;
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
                return [];
            default:
                return state || [];
        }
    };
    var item = function (state, action) {
        if (state === void 0) { state = {}; }
        switch (action.type) {
            case "SET_" + singular_1.toSingularAction(nodeName):
                return action.payload;
            case "PATCH_" + singular_1.toSingularAction(nodeName):
                return tslib_1.__assign(tslib_1.__assign({}, state), action.payload);
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                return tslib_1.__assign(tslib_1.__assign({}, state), { items: listItems(state.items, action) });
            default:
                return state;
        }
    };
    var items = function (state, action) {
        var _a;
        if (state === void 0) { state = {}; }
        var newState;
        switch (action.type) {
            case "SET_" + nodeName.toUpperCase():
                return action.payload;
            case "SET_" + singular_1.toSingularAction(nodeName):
            case "PATCH_" + singular_1.toSingularAction(nodeName):
            case "SET_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_" + nodeName.toUpperCase() + "_ITEM":
            case "POP_" + nodeName.toUpperCase() + "_ITEM":
            case "CLEAR_" + nodeName.toUpperCase() + "_ITEMS":
            case "PUSH_MANY_" + nodeName.toUpperCase() + "_ITEMS":
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                return tslib_1.__assign(tslib_1.__assign({}, state), (_a = {}, _a[action.payload.id] = item(state[action.payload.id], action), _a));
            case "DELETE_" + singular_1.toSingularAction(nodeName):
                if (!action.payload || !action.payload.id) {
                    return state;
                }
                newState = tslib_1.__assign({}, state);
                delete newState[action.payload.id];
                return newState;
            case "SET_MANY_" + nodeName.toUpperCase():
                return tslib_1.__assign(tslib_1.__assign({}, state), action.payload);
            default:
                return state || {};
        }
    };
    return items;
};
exports.generateGroupedList = generateGroupedList;
var generateReducersForStore = function (storeState) {
    var output = {};
    var keys = Object.keys(storeState);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var type = structure_1.nodeToType(storeState[key]);
        switch (type) {
            case types_1.NodeType.SINGLE_NODE:
                output[key] = exports.generateSingle(String(key));
                break;
            case types_1.NodeType.QUEUE_NODE:
                output[key] = exports.generateQueue(String(key));
                break;
            case types_1.NodeType.COLLECTION_NODE:
                output[key] = exports.generateCollection(String(key));
                break;
            case types_1.NodeType.GROUPED_LIST_NODE:
                output[key] = exports.generateGroupedList(String(key));
                break;
        }
    }
    return output;
};
exports.generateReducersForStore = generateReducersForStore;
